#ifndef LOWLEVEL_H
#define LOWLEVEL_H
/*
* Dateiname.... lowlevel.h
* erstellt am.. 2016-10-30
* gendert am.. 2016-10-30
* Autor........ Clemens B Fahr
* email........ clemens.fahr[at]fahrnet.de
* Kommentar.... based on RFM_wireless_4CH_sensor_02_mod
* .............	
* .............				
*/

// includes
//------------------------------------------------------------------
#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/eeprom.h>
#include <inttypes.h>

// defines
//------------------------------------------------------------------
// clock prescaler: internal clock 8MHz => 2MHz f_CPU => divider 4
// activate if external clock speed needs to be reduced
#define CLK_PRESCALER (0x01 << CLKPS1)

// timer0 clock source for the UART bit delay routine
// at 2MHz clock yielding 
#define CLK_SRC_BIT_T0 (0x01 << CS00)
// timer0 counter preload value, count up from 200 to 256 gives overflow after 52us (used for soft-UART baud rate of 19.2kbaud)
#define PRELOAD_BIT_T0 200
#define TIMER0_CLK_OFF ~(0x01 << CS02) & ~(0x01 << CS01) & ~(0x01 << CS00)
// timer1 clock source for the 1ms-delay-routine
// at 2MHz clock use divider 16 yielding 125kHz
#define CLK_SRC_1MS_T1 ((0x01 << CS12) | (0x01 << CS10))
// timer1 counter preload value, count up from 131 to 256 gives overflow after 1ms
#define PRELOAD_1MS_T1	131

// timer1 clock source for the 100ms-delay-routine
// at 2MHz clock use divider 1024 yielding 1.95kHz
#define CLK_SRC_100MS_T1 ((0x01 << CS13) | (0x01 << CS11) | (0x01 << CS10))
// timer1 counter preload value, count up from 61 to 256 gives overflow after 100ms
#define PRELOAD_100MS_T1        63

// timer1 clock source for the 500ms-delay-routine
// at 2MHz clock use divider 8192 yielding 240Hz
#define CLK_SRC_500MS_T1 ((0x01 << CS13) | (0x01 << CS12) | (0x01 << CS11))
// timer1 counter preload value, count up from 134 to 256 gives overflow after 500ms
#define PRELOAD_500MS_T1	134
#define TIMER1_CLK_OFF ~(0x01 << CS13) & ~(0x01 << CS12) & ~(0x01 << CS11) & ~(0x01 << CS10)
#define USE_AREF 1
#define USE_INTERNAL_REF 0

// variables
//------------------------------------------------------------------
extern volatile uint8_t TX_buffer[];
extern uint8_t DEVICE_UID2;

// forward and external declarations
//------------------------------------------------------------------
void LOWLEVEL_init_all(void);
void LOWLEVEL_init_RFM(void);
void LOWLEVEL_sleep_RFM(uint16_t sleep_time);
void LOWLEVEL_init_ADC_buffer(uint8_t numba_bytes);
void LOWLEVEL_send_telegram_RFM(uint8_t NumbaDataBytes);
uint16_t LOWLEVEL_status_register_read(void);
void LOWLEVEL_delay_1ms(uint16_t multiplier);
void LOWLEVEL_delay_sleep_100ms(volatile uint8_t multiplier);
void LOWLEVEL_blink_status_LED(volatile uint8_t numba_blink);
uint16_t LOWLEVEL_adcConvert(uint8_t ADC_channel, uint8_t reference_voltage);
void LOWLEVEL_debugOut(uint8_t aByte);

uint16_t send_command_RFM(uint16_t aCommand);
void UART_bit_delay(void);
void run_timer0(uint8_t preloadvalue, uint8_t clocksource, uint8_t IRQ_enabled);
void stop_timer0(void);
void run_timer1(uint8_t preloadvalue, uint8_t clocksource, uint8_t IRQ_enabled);
void stop_timer1(void);

#endif